#include "Service.h"

std::vector<std::vector<unsigned int>> const compoTeams = {{4,3,2},{5,3,2},{6,4,2},{7,5,3},{8,5,3}};
std::vector<std::vector<unsigned int>> const compoCartes = {{4,15,4,1},{5,19,5,1},{6,23,6,1},{7,27,7,1},{8,31,8,1}};

std::vector<Joueur> listeJoueurs = GJoueurs::getListeJoueurs();
std::vector<Carte> listeCartesRev = GCartes::getListeCartesRev();
std::vector<Carte> listeCartesRes = GCartes::getListeCartesRes();

std::string Service::debutPartie(unsigned int nbJoueur)
{
	unsigned int nbEnfant = 0;
	unsigned int nbDemon = 0;
	unsigned int nbCC = 0; 
	unsigned int nbHDMI = 0;
	unsigned int r1 = 0;
	unsigned int r2 = 0;

	//chercher le nombre de joueurs de chaque team
	for(unsigned int t = 0; t < compoTeams.size(); t++)
	{
		if(compoTeams[t][0] == nbJoueur)
		{
			if(nbJoueur == 4 || nbJoueur == 7)
			{
				r1 = rand() % 2;
				r2 = 1-r1;
			}
			nbEnfant = compoTeams[t][1]-r1;
			nbDemon = compoTeams[t][2]-r2;
		}
	}

	//chercher le nombre de cartes de chaque type
	for(unsigned int c = 0; c < compoCartes.size(); c++)
	{
		if(compoCartes[c][0] == nbJoueur)
		{
			nbCC = compoCartes[c][1];
			nbHDMI = compoCartes[c][2];
		}
	}

	//creer les enfants
	for(unsigned int i = 0; i < nbEnfant; i++)
		GJoueurs::creerEnfant("");

	//creer les demons
	for(unsigned int i = 0; i < nbDemon; i++)
		GJoueurs::creerDemon("");

	//creer les coupecoupes
	for(unsigned int i = 0; i < nbCC; i++)
		GCartes::creerCoupeCoupe();

	//creer les cables HDMI
	for(unsigned int i = 0; i < nbHDMI; i++)
		GCartes::creerHDMI();

	//creer le big ben
	GCartes::creerBigBen();

	//melanger la liste des joueurs et la liste des cartes
	std::random_shuffle(listeJoueurs.begin(), listeJoueurs.end());
	std::random_shuffle(listeCartesRes.begin(), listeCartesRes.end());

	//attribution aleatoire du joueur actif
	GJoueurs::setIdJoueurActif(rand() % listeJoueurs.size() + 1);

	//donner les cartes aux joueurs
	donnerCarte();

	//premier tour
	tour = 1;

	return "nbEnfant : " + std::to_string(nbEnfant) + "\nnbDemon : " + std::to_string(nbDemon) + "\nnbCoupeCoupe : " + std::to_string(nbCC) + "\nnbHDMI : " + std::to_string(nbHDMI);
}

std::string Service::annonceCarte(unsigned int idJoueur, unsigned int nbCC, unsigned int nbHDMI, unsigned int nbBB)
{
	return "idJoueur : " + std::to_string(idJoueur) + "\nnbCC : " + std::to_string(nbCC) + "\nnbHDMI : " + std::to_string(nbHDMI) + "\nnbBB : " + std::to_string(nbBB);
}

std::string Service::retournerCarte(unsigned int idJoueur, unsigned int idCarte)
{
	//si le joueur qui choisit la carte n'est pas le joueur actif
	if(idJoueur != GJoueurs::getIdJoueurActif())
		return "Ce n'est pas votre tour de jouer !";

	Carte* c = GCartes::getCarte(idCarte);

	//changement de joueur actif
	GJoueurs::setIdJoueurActif(getJoueurParCarte(idCarte));

	//si la carte choisie n'existe pas
	if(c == nullptr)
		return "Cette carte n'existe pas";

	//on retire la carte de la liste des cartes restantes 
	//on ajoute la carte dans la liste des cartes revelees
	for(unsigned int i = 0; i < listeCartesRes.size(); i++)
	{
		if(&listeCartesRes[i] == c)
		{
			listeCartesRev.push_back(*c);
			listeCartesRes.erase(listeCartesRes.begin() + i);
		}
	}

	//si la carte revelee est bigben alors la partie est finie
	if(c->getType() == "BigBen")
	{
		finPartie(1);
	}

	//si la carte revelee est un coupecoupe alors on regarde si toutes les cartes coupecoupes ont ete revele
	if(c->getType() == "CoupeCoupe")
	{
		unsigned int nbCC = 0;
		for(unsigned int i = 0; i < listeCartesRev.size(); i++)
		{
			if(listeCartesRev[i].getType() == "CoupeCoupe")
				nbCC++;
		}
		if(nbCC == listeJoueurs.size())
		{
			return finPartie(0);
		}
	}

	//si c'est la derniere carte du tour alors on passe au tour suivant
	if(listeCartesRev.size()%listeJoueurs.size() == 0)
		return tourSuivant();

	return "";
}

std::string Service::tourSuivant()
{
	if(++tour == 5)
		finPartie(1);
	else
		donnerCarte();

	return "Le tour " + std::to_string(tour) + " va commencer !";
}

std::string Service::finPartie(unsigned int victoire)
{	 
	if(victoire == 0){
		//gentils gagnent
		return "FIOU ! La bombe a été désamorcée, les pas méchants ont gagné";

	}
	else{
		//méchants gagnent
		return "BOOM ! Bigben a boomboom, les pas gentils ont gagné";
	}
}

std::string Service::donnerCarte()
{
	//melanger la liste des cartes restantes
	std::random_shuffle(listeCartesRes.begin(), listeCartesRes.end());

	//donner les cartes aux joueurs
	for(unsigned int i = 0; i < listeJoueurs.size(); i++)
	{
		listeJoueurs[i].getCartes().clear();
		for(unsigned int j = 0; j < 6-tour; j++)
			listeJoueurs[i].getCartes().push_back(listeCartesRes[i * (6-tour) + j]);
	}

	return "Les cartes ont été distribuées";
}

unsigned int getJoueurParCarte(unsigned int idCarte)
{
	for(unsigned int i = 0; i < listeJoueurs.size(); i++)
	{
		for(unsigned int j = 0; j < listeJoueurs[i].getCartes().size(); j++)
		{
			if(listeJoueurs[i].getCartes()[j].getId() == idCarte)
				return listeJoueurs[i].getId();
		}
	}

	return 0;
}